/***************************************************************************
 * Copyright 2001-2009 The VietSpider         All rights reserved.  		 *
 **************************************************************************/
package org.vietspider.offices;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.OutputStreamWriter;
import java.io.Writer;

import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.util.PDFText2HTML;
import org.apache.pdfbox.util.PDFTextStripper;
import org.vietspider.common.Application;
import org.vietspider.common.Install;
import org.vietspider.common.io.DataWriter;
import org.vietspider.common.io.UtilFile;
import org.vietspider.html.util.HyperLinkUtil.SiteLinkVerifier;

/** 
 * Author : Nhu Dinh Thuan
 *          nhudinhthuan@yahoo.com
 * Aug 13, 2009  
 */
public class DocumentConverter {
  
  private volatile static DocumentConverter instance = null;
  
 /* public static synchronized  void createInstance(String path) {
    try {
      instance = new DocumentConverter(path);
    } catch (Exception e) {
      LogService.getInstance().setThrowable(e);
      instance = null;
    }
  }*/
  
  public static DocumentConverter getInstance() {
    if(Application.LICENSE == Install.PERSONAL) return null;
    if(instance == null) instance = new DocumentConverter();
    return instance; 
  }
  
  private final static String [] extensions = {
    "pdf"
//     "doc",  "rtf", 
//     "odt", "ott", "sxw", "stw", "sdw", "psw",
//     "ppt", "pot", "pps", "ppt", "ppz",
//     "xlc", "xll", "xlm", "xls", "xlw",
  };
  
  private final static String[][] mimeTypes = {
    {"application/pdf", "pdf"}
//    {"application/msword", "doc"}, {"text/rtf", "rtf"},
//    {"application/vnd.ms-excel", "xls"},  {"application/mspowerpoint", "ppt"}, 
//    {"application/vnd.oasis.opendocument.text", "odt"}, 
//    {"application/vnd.oasis.opendocument.spreadsheet", "ods"},
//    {"application/vnd.oasis.opendocument.presentation", "odp"},
//    {"application/vnd.oasis.opendocument.graphics", "odg"}
    
  };
  

  private DocumentConverter() {
  }
  
  public byte[] convert(/*String id, */byte [] data, String ext, String encoding) throws Exception {
    ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
    PDDocument document = null;
    Writer output = null;
    try {
      ByteArrayInputStream input = new ByteArrayInputStream(data);
      document = PDDocument.load(input, false);

      PDFTextStripper stripper = new PDFText2HTML(encoding);
      stripper.setForceParsing(false);
      stripper.setSortByPosition(false);
      stripper.setShouldSeparateByBeads( true);
      stripper.setStartPage(1);
      stripper.setEndPage(Integer.MAX_VALUE);

      output = new OutputStreamWriter(outputStream, encoding);
      stripper.writeText(document, output);
    } finally {
      if(output != null) output.close();
      if(document != null)  document.close();
    }
    
    byte[] bytes = outputStream.toByteArray();
    
    //only for test
    String id = String.valueOf(System.currentTimeMillis());
    File file = new File(UtilFile.getFolder("track/temp/offices/"), id + "." + ext);
    new DataWriter().save(file, data);
    
    return bytes;
//    File file = UtilFile.getFile("track/temp/offices/"+id, "input."+ ext);
//    new DataWriter().save(file, data);
//    return convert(id, file);
  }
  
//  public synchronized byte[] convert(String id, File file) throws Exception {
//    return null;
//  }
  
  public String getExtension(String link, String contentType) {
    if(link != null) {
      String url = link.toLowerCase();
      for(int i = 0; i < extensions.length; i++) {
        if(url.endsWith(extensions[i]) 
            && url.charAt(url.length() - 4) == '.') return extensions[i];
      }
    }
    
    if(contentType == null) return null;
    
    for(int i = 0; i < mimeTypes.length; i++) {
      if(mimeTypes[i][0].equalsIgnoreCase(contentType)) return mimeTypes[i][1];
    }
    
    return null;
  }
  
  public static class FullSiteLinkVerifier extends SiteLinkVerifier {
    public boolean verify(String link){
      if(link == null) return false;
      link = link.toLowerCase();    
      String start[]={"mailto", "javascript", "window", "history", "#"};
      String end[]={"css", "js", "jpg", "png", "gif", "jpeg", "bmp", "dat", "exe",
                    "doc", "xls",
//                    "pdf", 
                    "rm", "ram", "wma", "wmv", "mp3", "swf", "zip", "jar", "rar"};
      String exist[] ={"img(\"", "image", ":sendim"} ;
      return !startOrEndOrExist(link, start, end, exist); 
    }
  }
  
}
